# ttk_helper.tcl
# Tcl procedures for Stratix10 TX

puts {==============================================================================================================}
puts {This tcl file contains a number of procedures to do additional configuration and status updates for Stratix 10} 
puts {This version of the script is only for E-Tile}
puts {Revision 1.13}
puts {Date : 12 May 2020}
puts {Author : Peter Schepers}
puts {==============================================================================================================}
puts ""

# Offset NRZ + PAM4 using single channel
set offset_etile 0x200000

# Offset High datarate
set offset_etile_high_datarate 0x400000

#for high_datarate PAM4 set this to 1
set high_datarate 0 

#set to 1 for test_mode (note that this expects the 48 or 96 channel design in terms of indexes for the phy)
set test_mode 0

###############################################################################
# Map all and claim all Etile Slaves
# Once claimed they can be used using the $phy_key
###############################################################################
set ETILE_SLAVE_MAP {}

proc map_all_etile_slaves {dump_map} {
    global ETILE_SLAVE_MAP
    set num_slave_services [llength [get_service_paths slave]]
    set array_list {}
	 puts "Below is the list of E-tile PHY's claimed"
	 puts "Use etile_phy_<i> as phy identifier in the functions"
    for {set i 0} {$i < $num_slave_services} {incr i} {
        catch {
            set marker_info [marker_get_info [lindex [get_service_paths slave] $i]]
            regexp {TYPE_NAME(.+)FULL_HPATH} $marker_info full_match matched_typename
            regexp {FULL_HPATH(.+)BASE_ADDRESS} $marker_info full_match matched_hpath
            if {[string match "*altera_xcvr_native_s10_etile.slave*" $matched_typename]} {
                set claimed_slave_value [claim_service slave [lindex [get_service_paths slave] $i] ""]
					 set slave_addr [dict get $marker_info BASE_ADDRESS]
                set phy_key "etile_phy_$i"
                lappend array_list $phy_key
                lappend array_list $claimed_slave_value
                
                if {$dump_map} {
                    puts "Phy : \"$phy_key\" ---> Hierarchy Path: $matched_hpath   Slave: [format 0x%x $slave_addr]"					  
                }
            }
        }
    }
	 puts ""
    set ETILE_SLAVE_MAP $array_list
}

if {$test_mode == 1} {
map_all_etile_slaves 1
}



###############################################################################
# read register
###############################################################################
proc rd {m base_addr offset } {
  set value [master_read_8 $m [expr $base_addr + $offset * 4] 1]

}
#puts {rd m base_addr offset }
#puts {example usage: rd $m $base_addr 0x138}
#puts ""

###############################################################################
# write register
###############################################################################
proc wr {m base_addr offset newval} {
  # write the new data back
  master_write_8 $m [expr $base_addr + $offset * 4] $newval
}


proc rd_mask_dec {m base_addr offset bitmask } {
  set value [master_read_8 $m [expr $base_addr + $offset * 4] 1]
  puts -nonewline [format %d [expr $value & $bitmask]]
}

proc rd_mask_hex {m base_addr offset bitmask } {
  set value [master_read_8 $m [expr $base_addr + $offset * 4] 1]
  puts -nonewline [format 0x%x [expr $value & $bitmask]]
}


###############################################################################
# read modify write register
###############################################################################

proc rmw {m base_addr offset bitmask newval} {
  # mask out the newval so that un-masked bits do not change
  set newval_masked [expr $newval & $bitmask]

  # read from data register
  set value [master_read_8 $m [expr $base_addr + $offset * 4] 1]
  
  # bitwise-AND with the negative of bitmask to clear out only the bitmask bits
  set value [expr $value & [expr 0xff & ~$bitmask]] 

  # bitwise-OR this data with the newval_masked so that only the masked bits get changed
  set value [expr $value | $newval_masked] 

  # write the new data back
  master_write_8 $m [expr $base_addr + $offset * 4] $value


}
#puts {rmw {m base_addr offset bitmask newval} }
#puts {example usage: rmw $m $base_addr 0x160 0xE 0x4}
#puts ""



	
###############################################################################		
#Do rmw for all channels 
###############################################################################
proc rmw_all {offset mask newval verbose} {
global ETILE_SLAVE_MAP
global offset_etile
global offset_etile_high_datarate
global high_datarate


set length [llength $ETILE_SLAVE_MAP]
puts $length 

	for {set i 0} {$i < $length/2} {incr i} {
	
	set phy [lindex $ETILE_SLAVE_MAP [expr $i * 2]]
	#puts $phy
				
			  if {$high_datarate == 0} {
					set number_of_channels [rd_channel $phy 0 0x40010 0]
				} else {
					set temp [rd_channel $phy 0 0x40010 0]
					set number_of_channels [expr $temp/2]
					#puts $number_of_channels
					}

					

			# Loop through all channels in this Native PHY
			for {set channel 0} {$channel < $number_of_channels} {incr channel} {
					


				rmw_channel $phy $channel $offset $mask $newval 0

				
				
				# Print info
				if {$verbose == 1} {						
				puts -nonewline "Phy: $phy  "
				puts -nonewline "channel  : $channel	"
				puts -nonewline "	Offset : [format 0x%x $offset] "
			   }	
				set read_back [rd_channel $phy $channel $offset 0]
				set read_back_mask [expr [expr $read_back & $mask ]]
				if {$verbose == 1} {		
				puts -nonewline "	Newval : [format 0x%x $read_back_mask] " 
				puts ""
				}
}}}


puts {rmw_all {offset mask newval verbose}}
puts {example useage rmw_all 0x160 0x0E 0x03 1}
puts ""


###############################################################################		
#Do rmw for specific channel within a phy
###############################################################################
proc rmw_channel {phy channel offset mask newval verbose} {
global offset_etile
global offset_etile_high_datarate
global high_datarate

global ETILE_SLAVE_MAP
array set etile_slave_array $ETILE_SLAVE_MAP
set m $etile_slave_array($phy)

				 if {$high_datarate == 0} {
					set base_addr [expr $offset_etile * $channel]
				} else {
					set base_addr [expr $offset_etile_high_datarate * $channel]
					}
		
				rmw $m $base_addr $offset $mask $newval

				
				
				# Print info
				if {$verbose == 1} {				
				puts -nonewline "Slave: $phy  "
				puts -nonewline "channel  : $channel	"
				puts -nonewline "base_addr : [format 0x%x $base_addr]	"
				puts -nonewline "	Offset : [format 0x%x $offset] " 
				}
				set read_back [rd $m $base_addr $offset]
				set read_back_mask [expr [expr $read_back & $mask ]]
				if {$verbose == 1} {						
				puts -nonewline "	Newval : [format 0x%x $read_back_mask] " 
				puts ""
				}

}
puts {rmw_channel {phy channel offset mask newval verbose}}
puts {example useage rmw_channel etile_phy_33 1 0x40000 0x0F 0x03 1}
puts ""

###############################################################################		
#write to a channel channel
###############################################################################
proc wr_channel {phy channel offset newval verbose} {
global offset_etile
global offset_etile_high_datarate
global high_datarate

global ETILE_SLAVE_MAP
array set etile_slave_array $ETILE_SLAVE_MAP
set m $etile_slave_array($phy)

				 if {$high_datarate == 0} {
					set base_addr [expr $offset_etile * $channel]
				} else {
					set base_addr [expr $offset_etile_high_datarate * $channel]
					}
		
				wr $m $base_addr $offset $newval

				
				
				# Print info
				if {$verbose == 1} {				
				puts -nonewline "Slave: $phy  "
				puts -nonewline "channel  : $channel	"
				puts -nonewline "base_addr : [format 0x%x $base_addr]	"
				puts -nonewline "	Offset : [format 0x%x $offset] " 
				}
				if {$verbose == 1} {						
				puts -nonewline "	Newval : [format 0x%x $newval] " 
				puts ""
				}

}
puts {wr_channel {phy channel offset newval verbose}}
puts {example useage wr_channel etile_phy_33 1 0x207 0x91 1}
puts ""


###############################################################################		
#read a register from a specific channel in a phy
###############################################################################
proc rd_channel {phy channel offset verbose} {
global offset_etile
global offset_etile_high_datarate
global high_datarate

global ETILE_SLAVE_MAP
array set etile_slave_array $ETILE_SLAVE_MAP
set m $etile_slave_array($phy)
	 
				 if {$high_datarate == 0} {
					set base_addr [expr $offset_etile * $channel]
				} else {
					set base_addr [expr $offset_etile_high_datarate * $channel]
					}

				
				
				# Print info
				set identifier [rd $m 0x0 0x40000]
				if {$verbose == 1} {
				puts -nonewline "Slave: $phy  "
				#puts -nonewline "Identifier: $identifier  "
				puts -nonewline "channel  : $channel	"
				puts -nonewline "base_addr : [format 0x%x $base_addr]	 "
				puts -nonewline "Offset : [format 0x%x $offset]  " 
				}
				set read_back [rd $m $base_addr $offset]
				if {$verbose == 1} {				
				puts -nonewline "	Read : [format 0x%x $read_back] " 
				puts ""
				}
return $read_back
}
puts {rd_channel {phy channel offset verbose}}
puts {to read and printout use e.g. rd_channel etile_phy_33 3 0x40000 1}
puts {when only reading without printing use e.g. set d [rd_channel etile_phy_33 3 0x40000 0]}
puts ""
if {$test_mode == 1} {
rd_channel etile_phy_33 1 0x40000 1
}

###############################################################################		
#Do read of a register for all channels
###############################################################################
proc rd_all {offset} {
global ETILE_SLAVE_MAP
global offset_etile
global offset_etile_high_datarate
global high_datarate


set length [llength $ETILE_SLAVE_MAP]
puts $length 

	for {set i 0} {$i < $length/2} {incr i} {
	
	set phy [lindex $ETILE_SLAVE_MAP [expr $i * 2]]
	#puts $phy
				
			  if {$high_datarate == 0} {
					set number_of_channels [rd_channel $phy 0 0x40010 0]
				} else {
					set temp [rd_channel $phy 0 0x40010 0]
					set number_of_channels [expr $temp/2]
					#puts $number_of_channels
					}

					

			# Loop through all channels in this Native PHY
			for {set channel 0} {$channel < $number_of_channels} {incr channel} {
		


				
				
				# Print info

				puts -nonewline "Phy: $phy  "
				puts -nonewline "channel  : $channel	"
				puts -nonewline "	Offset : [format 0x%x $offset] " 
				set read_back [rd_channel $phy $channel $offset 0]
				puts -nonewline "	ReadValue : [format 0x%x $read_back] " 
				puts ""
}}}


puts {rd_all {offset}}
puts {example useage rd_all 0x40011}
puts ""
if {$test_mode == 1} {
rd_all 0x40011
}

###############################################################################
#raw dump of channel
###############################################################################

proc raw_dump_channel {phy ch start_addr stop_addr} {
global offset_etile
global offset_etile_high_datarate
global high_datarate

global ETILE_SLAVE_MAP
array set etile_slave_array $ETILE_SLAVE_MAP
set m $etile_slave_array($phy) 
		
				if {$high_datarate == 0} {
					set base_addr [expr $offset_etile * $ch]
				} else {
					set base_addr [expr $offset_etile_high_datarate * $ch]
					}
	
set addr $start_addr

	for {set i 0} {$i <= [expr {$stop_addr - $start_addr}]} {incr i} {
		set d1 [rd $m $base_addr $addr]
		#puts $d1
		#puts $d2
			puts [format "ADDR: 0x%03x --> ch%d:  0x%02x" $addr $ch $d1 ]  
		incr addr
	}
}

puts {raw_dump_channel {phy ch start_addr stop_addr}}
puts {example useage raw_dump_channel etile_phy_33 0 0x000 0x300}
puts ""
if {$test_mode == 1} {
raw_dump_channel etile_phy_33 0 0x000 0x300
}

###############################################################################
#Compare differences between channels
###############################################################################

proc compare_xcvr_channels {phy ch1 ch2 start_addr stop_addr} {
global offset_etile
global offset_etile_high_datarate
global high_datarate

global ETILE_SLAVE_MAP
array set etile_slave_array $ETILE_SLAVE_MAP
set m $etile_slave_array($phy)
		
				if {$high_datarate == 0} {
					set base_addr1 [expr $offset_etile * $ch1]
				} else {
					set base_addr1 [expr $offset_etile_high_datarate * $ch1]
					}

				if {$high_datarate == 0} {
					set base_addr2 [expr $offset_etile * $ch2]
				} else {
					set base_addr2 [expr $offset_etile_high_datarate * $ch2]
					}
					
	
set addr $start_addr

	for {set i 0} {$i < [expr {$stop_addr - $start_addr}]} {incr i} {
		set d1 [rd $m $base_addr1 $addr]
		set d2 [rd $m $base_addr2 $addr]
		#puts $d1
		#puts $d2
		if {[expr {$d1 ne $d2}] } {
			puts [format "ADDR: 0x%03x --> ch%d:  0x%02x   /   ch%d: 0x%02x" $addr $ch1 $d1 $ch2 $d2]  
			}
		incr addr
	}
}

puts {compare_xcvr_channels {phy ch1 ch2 start_addr stop_addr}}
puts {example useage compare_xcvr_channels etile_phy_33 0 1 0x0 0xF}
puts ""
if {$test_mode == 1} {
compare_xcvr_channels etile_phy_33 0 1 0x0 0xF
}


###############################################################################
#Reconfig Etile
###############################################################################


proc rcfg_etile {phy channel interrupt_code interrupt_data verbose} {
global offset_etile
global offset_etile_high_datarate
global high_datarate

global ETILE_SLAVE_MAP
array set etile_slave_array $ETILE_SLAVE_MAP
set m $etile_slave_array($phy)
			
				if {$high_datarate == 0} {
					set base_addr [expr $offset_etile * $channel]
				} else {
					set base_addr [expr $offset_etile_high_datarate * $channel]
					}
					

			
 master_write_8 $m [expr $base_addr + 0x84 * 4] [expr $interrupt_data & 0xff]
 master_write_8 $m [expr $base_addr + 0x85 * 4] [expr $interrupt_data>>8 & 0xff]
 master_write_8 $m [expr $base_addr + 0x86 * 4] [expr $interrupt_code & 0xff]
 master_write_8 $m [expr $base_addr + 0x87 * 4] [expr $interrupt_code>>8 & 0xff]
 master_write_8 $m [expr $base_addr + 0x90 * 4] 0x1

 
 set inprog [master_read_8 $m [expr $base_addr + 0x8a * 4] 1]
 set inprog [expr $inprog & 0x80]

 set value 0
 if { $inprog == 0x80 } {
     while {$value != 0x0} {
         set value  [master_read_8 $m [expr $base_addr + 0x8b * 4] 1]
         set value  [expr $value & [expr 0x01]]
     }
 } else {
     puts "ERROR when set the Core PMA register read/write"
 }


 master_write_8 $m [expr $base_addr + 0x8A * 4] 0x80
 #return $value
 

set read_a [rd $m $base_addr 0x88]
#puts $read_a
set read_b [rd $m $base_addr 0x89]
#puts $read_b
set read_back [expr ($read_b << 8)  + $read_a ]

if {$verbose == 1} {
puts -nonewline "Slave: $phy  "
puts -nonewline "channel  : $channel	"
puts -nonewline "base_addr : [format 0x%x [expr $base_addr ]]	"
puts -nonewline "readback value : [format 0x%x [expr $read_back ]]	"
puts ""
	}
return $read_back	
}
puts {rcfg_etile {phy channel interrupt_code interrupt_data verbose}}
puts {example useage reading vod : set readback_vod [rcfg_etile etile_phy_33  0 0x15 0x4100 1]}
puts {example useage writing vod setting 3 : set readback_vod	[rcfg_etile etile_phy_33 0 0x15 0x4003 0]}
puts ""
if {$test_mode == 1} {
set readback_vod [rcfg_etile etile_phy_33  0 0x15 0x4100 1]
set readback_vod	[rcfg_etile etile_phy_33 0 0x15 0x4003 0]
}









###############################################################################
#Set Loopbacks (serial and reverse parallel)
###############################################################################

proc set_loopback_etile {phy channel serial_loop rev_parallel verbose} {
global offset_etile
global offset_etile_high_datarate
global high_datarate

		  if {($serial_loop == 1) && ($rev_parallel == 1)} {
		   puts -nonewline "Illegal setting, it is not possible to set both serial and reverse parallel loopback"
			} elseif {$serial_loop == 1} {
					set readback  [rcfg_etile $phy  $channel 0x08 0x301 0]
						if {$verbose == 1} {
						puts -nonewline "Phy : $phy  "
						puts -nonewline "channel  : $channel	"
						puts -nonewline "	Serial Loop : $serial_loop" 
						puts ""	
						}					
			} elseif {$rev_parallel == 1} {
					set readback  [rcfg_etile $phy  $channel 0x08 0x310 0]	
						if {$verbose == 1} {
						puts -nonewline "Phy : $phy  "
						puts -nonewline "channel  : $channel	"
						puts -nonewline "	Reverse Parallel Loop : $rev_parallel" 
						puts ""	
						}					
				} else {
					set readback  [rcfg_etile $phy  $channel 0x08 0x300 0]
						if {$verbose == 1} {
						puts -nonewline "Phy : $phy  "
						puts -nonewline "channel  : $channel	"
						puts -nonewline "	Serial Loop : $serial_loop" 				
						puts -nonewline "	Reverse Parallel Loop : $rev_parallel" 
						puts ""	
						}					
			}	



}
puts {set_loopback_etile {phy channel serial_loop rev_parallel verbose}}
puts {example to set serial loopback on channel 1 of etile_phy_33 :  set_serial_loopback_etile etile_phy_33 1 1 0 1}
puts ""
if {$test_mode == 1} {
set_loopback_etile etile_phy_33 1 1 0 1
}





###############################################################################
#Set Mode Etile
###############################################################################

proc set_mode_etile {phy channel rx_termination line_encoding tx_clk_divider rx_clk_divider gray_encoding encoding_1_1plusd verbose} {
global offset_etile
global offset_etile_high_datarate
global high_datarate


#1.	Disable xcvr first, s10e_interrupt_request $m $base_addr 0x01 0x0	
#	readback = rcfg_etile (phy,(SelectedChannel << CHANNEL_OFFSET), 0x01,0x00,0x0001,1);
		set readback  [rcfg_etile $phy  $channel 0x01 0x00 0]  
		
#Set termination (if external AC-coupling capacitors are used set RX termination to VCC(1), when no external AC-coupling capacitors are used set Rx termination to floating (2))
#	readback = rcfg_etile (phy,(SelectedChannel << CHANNEL_OFFSET), 0x2B,rx_termination,0x2B,1);	
		set readback  [rcfg_etile $phy  $channel 0x2B $rx_termination 0] 	

#2.	Change to tx clk divider, 300MHz, 30Gbps PAM4, s10e_interrupt_request $m $base_addr 0x05 0x32	
#	if ((mode == PAM4) && (high_datarate == 0))
#			readback = rcfg_etile (phy,(SelectedChannel << CHANNEL_OFFSET), 0x05,tx_clk_divider/2,0x0005,1);
#	else
#		readback = rcfg_etile (phy,(SelectedChannel << CHANNEL_OFFSET), 0x05,tx_clk_divider,0x0005,1);

		if {($line_encoding == "PAM4") && ($high_datarate == 0)} {
			set readback  [rcfg_etile $phy  $channel 0x05 [expr $tx_clk_divider/2] 0]
		} else {
			set readback  [rcfg_etile $phy  $channel 0x05 $tx_clk_divider 0] 		
		}
		
#3.	Change  rx clk divider, s10e_interrupt_request $m $base_addr 0x06 0x32
#	if ((mode == PAM4) && (high_datarate == 0))	
#	   readback = rcfg_etile (phy,(SelectedChannel << CHANNEL_OFFSET), 0x06,rx_clk_divider/2,0x0006,1);
#	else
#	   readback = rcfg_etile (phy,(SelectedChannel << CHANNEL_OFFSET), 0x06,rx_clk_divider,0x0006,1);

		if {($line_encoding == "PAM4") && ($high_datarate == 0)} {
			set readback  [rcfg_etile $phy  $channel 0x06 [expr $rx_clk_divider/2] 0]
		} else {
			set readback  [rcfg_etile $phy  $channel 0x06 $rx_clk_divider 0] 		
		}
	

#4.	set encoding and bit width, s10e_interrupt_request $m $base_addr 0x14 0x00dd
#	if (mode == PAM4)
#	{
#		if (high_datarate == 0)
#			readback = rcfg_etile (phy,(SelectedChannel << CHANNEL_OFFSET), 0x14,0x00DD,0x0014,1); //PAM4 32-bit
#		else
#			readback = rcfg_etile (phy,(SelectedChannel << CHANNEL_OFFSET), 0x14,0x00EE,0x0014,1); //PAM4 64-bit
#	}
#	else //NRZ
#	{
#		if (high_datarate == 0)
#			readback = rcfg_etile (phy,(SelectedChannel << CHANNEL_OFFSET), 0x14,0x0055,0x0014,1); //NRZ 32-bit
#		else
#			readback = rcfg_etile (phy,(SelectedChannel << CHANNEL_OFFSET), 0x14,0x0066,0x0014,1); //NRZ 64-bit 
#	}

		if {$line_encoding == "PAM4"} {
			if {$high_datarate == 0} {
			set readback  [rcfg_etile $phy  $channel 0x14 0x00DD 0]
			} else {
			set readback  [rcfg_etile $phy  $channel 0x14 0x00EE 0]
			}
		} else {
			if {$high_datarate == 0} {		
				set readback  [rcfg_etile $phy  $channel 0x14 0x0055 0] 
			} else {
			   set readback  [rcfg_etile $phy  $channel 0x14 0x0066 0] 
			}
		}

	
# 4b. Change encoding to gray + 1D encoding/decoding.(0x84[1] and 0x84[5]) attribute code 0x13 and 0x85[3..2])
#	
#	//Set gray encoding
#   if ((gray_encoding == 1) && (encoding_1_1plusd == 0))
#		readback = rcfg_etile (phy,(SelectedChannel << CHANNEL_OFFSET), 0x13,0x0C22,0x0013,1);
#	else if ((gray_encoding == 1) && (encoding_1_1plusd == 1))
#		readback = rcfg_etile (phy,(SelectedChannel << CHANNEL_OFFSET), 0x13,0x0C66,0x0013,1);	
#	else if ((gray_encoding == 0) && (encoding_1_1plusd == 1)) // not sure if this is relevant
#		readback = rcfg_etile (phy,(SelectedChannel << CHANNEL_OFFSET), 0x13,0x0C44,0x0013,1);	
#	else  // off
#		readback = rcfg_etile (phy,(SelectedChannel << CHANNEL_OFFSET), 0x13,0x0C00,0x0013,1);

		if {($gray_encoding == 1) && ($encoding_1_1plusd == 0)} {
			set readback  [rcfg_etile $phy  $channel 0x13 0x0C22 0]
		} elseif {($gray_encoding == 1) && ($encoding_1_1plusd == 1)} {
			set readback  [rcfg_etile $phy  $channel 0x13 0x0C66 0] 		
		} elseif {($gray_encoding == 0) && ($encoding_1_1plusd == 1)} {
			set readback  [rcfg_etile $phy  $channel 0x13 0x0C44 0] 		
		} else {
			set readback  [rcfg_etile $phy  $channel 0x13 0x0C00 0] 		
		}		

# 4c.	
#	//Tx/RX recalibration Sets whether the PMA transmitter path, receiver path, or both are calibrated the next
#  // time the PMA is enabled when using PMA attribute code 0x0001.
#	//s10e_interrupt_request $m $base_addr 0x11 0x03 
#	readback = rcfg_etile (phy,(SelectedChannel << CHANNEL_OFFSET), 0x11,0x03,0x0000,0);

     set readback  [rcfg_etile $phy  $channel 0x11 0x03 0]
	

# 5.	Enable xcvr,     s10e_interrupt_request $m $base_addr 0x01 0x3
#	readback = rcfg_etile (phy,(SelectedChannel << CHANNEL_OFFSET), 0x01,0x07,0x0001,1);	
		set readback  [rcfg_etile $phy  $channel 0x01 0x07 0] 

				if {$verbose == 1} {	
				puts -nonewline "Phy : $phy  "
				puts -nonewline "channel  : $channel	"
				if {$line_encoding == "PAM4" } {
					puts -nonewline "	PAM4 " 
				} else {
					puts -nonewline " NRZ "
				}
				puts -nonewline "	Gray Encoding : $gray_encoding " 
				puts -nonewline "	1/1+D Encoding : $encoding_1_1plusd " 
				puts -nonewline "	TxClk Divider : $tx_clk_divider " 
				puts -nonewline "	RxClk Divider : $rx_clk_divider " 		
				puts ""
				}		
		

}
puts {set_mode_etile {phy channel rx_termination line_encoding tx_clk_divider rx_clk_divider gray_encoding encoding_1_1plusd verbose}}
if {$high_datarate == 0} {
puts {example to set PAM4 on channel 1 of etile_phy_33 with gray_encoding and rx_termination set to VCC : set_mode_etile etile_phy_33 1 1 PAM4 0x32 0x32 1 0 1}
} else {
puts {example to set PAM4 on channel 1 of etile_phy_33 with gray_encoding and rx_termination set to VCC : set_mode_etile etile_phy_33 1 1 PAM4 0x64 0x64 1 0 1}
}
puts ""
if {$test_mode == 1} {
set_mode_etile etile_phy_33 1 1 PAM4 0x64 0x64 1 0 1
}

###############################################################################
#iadp Single Channel
###############################################################################

proc iadp_channel {phy channel adaptation_mode verbose} {
global offset_etile
global offset_etile_high_datarate
global high_datarate


#	  //Configure PMA register read/write code 0x0A as following to enable initial coarse adaptive equalization:  
#	readback = rcfg_etile (phy,(SelectedChannel << CHANNEL_OFFSET), 0x0A,0x01,0x000A,1);	
	
		set t0 [clock clicks -millisec] 	
		
		
				set readback  [rcfg_etile $phy  $channel 0x2C 0x118 0] 
#				Effort 1				
				if {$adaptation_mode == 1} {
					set readback  [rcfg_etile $phy  $channel 0x6C 0x01 0] 	
				}
#				Effort 0.5				
				if {$adaptation_mode == 0.5} {				
					set readback  [rcfg_etile $phy  $channel 0x6C 0x10 0] 	
				}
#				Effort 0 :	
				if {$adaptation_mode == 0} {	
					set readback  [rcfg_etile $phy  $channel 0x6C 0x00 0] 
				}		

		set readback  [rcfg_etile $phy  $channel 0x0A 0x01 0] 
		
				if {$verbose == 1} {	
				puts -nonewline "Phy : $phy  "
				puts -nonewline "channel  : $channel	"
				puts -nonewline "	iadp Started" 
				puts ""
				}

#	 // Read the initial coarse adaptive equalization status
#	
#	usleep(100);
#	
#	do
#	{
#			readback = rcfg_etile (phy,(SelectedChannel << CHANNEL_OFFSET), 0x126,0xB00,0x0000,0);			
#		   adaptation_busy = 0xF & readback;  
#	}
#	// 5. Repeat Step 4 until readback value lower 4 bits are 0.
#	while (adaptation_busy > 0);


		after 100
		
		
 set adaptation_busy 1
     while {$adaptation_busy != 0x0} {
         set readback  [rcfg_etile $phy  $channel 0x0126 0xB00 0] 
         set adaptation_busy [expr 0xf & $readback]
     }		
			if {$verbose == 1} {
				puts -nonewline "Phy : $phy  "
				puts -nonewline "channel  : $channel	"
				puts -nonewline "	iadp Finished after " 
			   puts -nonewline "[expr {([clock clicks -millisec]-$t0)/1000.}] sec" 	
				puts ""	
			}
	

}
puts {iadp_channel {phy channel adaptation_mode verbose}}
puts {example to run 0.5 effort iadp on channel 1 of etile_phy_33   iadp_channel etile_phy_33 1 0.5 1}
puts ""
if {$test_mode == 1} {
iadp_channel etile_phy_33 1 0.5 1
}

###############################################################################
#iadp All channels in one Native PHY
###############################################################################

proc iadp_phy {phy adaptation_mode verbose} {
global offset_etile
global offset_etile_high_datarate
global high_datarate

				if {$verbose == 1} {	
				puts -nonewline "Phy : $phy  "
				puts -nonewline "	iadp Started on All channels" 
				puts ""
				}	

				 if {$high_datarate == 0} {
					set number_of_channels [rd_channel $phy 0 0x40010 0]
				} else {
					set temp [rd_channel $phy 0 0x40010 0]
					set number_of_channels [expr $temp/2]
					#puts $number_of_channels
					}
	


#	  //Configure PMA register read/write code 0x0A as following to enable initial coarse adaptive equalization:  
#	readback = rcfg_etile (phy,(SelectedChannel << CHANNEL_OFFSET), 0x0A,0x01,0x000A,1);	
	
		set t0 [clock clicks -millisec] 	
		
	for {set channel 0} {$channel < $number_of_channels} {incr channel} {

	

				set readback  [rcfg_etile $phy  $channel 0x2C 0x118 0] 
#				Effort 1				
				if {$adaptation_mode == 1} {
					set readback  [rcfg_etile $phy  $channel 0x6C 0x01 0] 	
				}
#				Effort 0.5				
				if {$adaptation_mode == 0.5} {				
					set readback  [rcfg_etile $phy  $channel 0x6C 0x10 0] 	
				}
#				Effort 0 :	
				if {$adaptation_mode == 0} {	
					set readback  [rcfg_etile $phy  $channel 0x6C 0x00 0] 
				}

	
		set readback  [rcfg_etile $phy  $channel 0x0A 0x01 0] 
		
	}


				
#	 // Read the initial coarse adaptive equalization status
#	
#	usleep(100);
#	
#	do
#	{
#			readback = rcfg_etile (phy,(SelectedChannel << CHANNEL_OFFSET), 0x126,0xB00,0x0000,0);			
#		   adaptation_busy = 0xF & readback;  
#	}
#	// 5. Repeat Step 4 until readback value lower 4 bits are 0.
#	while (adaptation_busy > 0);

		
	for {set channel 0} {$channel < $number_of_channels} {incr channel} {		
	set adaptation_busy 1
     while {$adaptation_busy != 0x0} {
         set readback  [rcfg_etile $phy  $channel 0x0126 0xB00 0] 
         set adaptation_busy [expr 0xf & $readback]
     }		
	}
			if {$verbose == 1} {
				puts -nonewline "Phy : $phy  "
				puts -nonewline "	iadp Finished on all [format %d $number_of_channels] channels after  " 
			   puts -nonewline "[expr {([clock clicks -millisec]-$t0)/1000.}] sec" 	
				puts ""	
			}
	

}
puts {iadp_phy {phy adaptation_mode verbose}}
puts {example to run iadp with 0.5 effort at once on all channels of etile_phy_33 :  iadp_phy etile_phy_33 0.5 1}
puts ""
if {$test_mode == 1} {
iadp_phy etile_phy_33 0.5 1
}

###############################################################################
#  Reset uC
###############################################################################


proc reset_uc {phy channel verbose} {
global offset_etile
global offset_etile_high_datarate
global high_datarate

				if {$verbose == 1} {	
				puts -nonewline "Phy : $phy  "
				puts -nonewline "Channel : $channel "				
				puts -nonewline "	Reset uC" 
				puts ""
				}	

#	wr_channel(phy, (channel << CHANNEL_OFFSET)+0x200,0x00);
#	wr_channel(phy, (channel << CHANNEL_OFFSET)+0x201,0x00);
#	wr_channel(phy, (channel << CHANNEL_OFFSET)+0x202,0x00);
#	wr_channel(phy, (channel << CHANNEL_OFFSET)+0x203,0x81);	  
	
	wr_channel $phy $channel 0x200 0x00 0
	wr_channel $phy $channel 0x201 0x00 0
	wr_channel $phy $channel 0x202 0x00 0
	wr_channel $phy $channel 0x203 0x81 0	
	
	
	set busy 0
     while {$busy != 0x80} {
         set readback  [rd_channel $phy $channel 0x207 0] 
         set busy [expr 0xff & $readback]
     }	

	#rmw_channel(phy, (channel << CHANNEL_OFFSET)+0x95,0x20,0x20);	 
   rmw_channel $phy $channel 0x95 0xFF 0x20 0	
		
	#Load original PMA settings (as stored in SOF file)
	#wr_channel(phy, (channel << CHANNEL_OFFSET)+0x91,0x01);
   wr_channel $phy $channel 0x91 0x01 0	


}
puts {reset_uc {phy channel verbose}}
puts {example to reset uc on channel 2 of  etile_phy_33 :  reset_uc etile_phy_33 1 1}
puts ""
if {$test_mode == 1} {
reset_uc etile_phy_33 1 1
}

###############################################################################
#  Reset uC all channels in a PHY
###############################################################################

proc reset_uc_phy_etile {phy verbose} {
global offset_etile
global offset_etile_high_datarate
global high_datarate

				if {$verbose == 1} {	
				puts -nonewline "Phy : $phy  "
				puts -nonewline "	Reset uC on all channels " 
				puts ""
				}	

				 if {$high_datarate == 0} {
					set number_of_channels [rd_channel $phy 0 0x40010 0]
				} else {
					set temp [rd_channel $phy 0 0x40010 0]
					set number_of_channels [expr $temp/2]
					}
	


		
	for {set channel 0} {$channel < $number_of_channels} {incr channel} {
	
		reset_uc $phy $channel 0

	}



}
puts {reset_uc_phy_etile {phy verbose}}
puts {example to reset uC on all channels of etile_phy_33 :  reset_uc_phy_etile etile_phy_33 1}
puts ""
if {$test_mode == 1} {
reset_uc_phy_etile etile_phy_33 1
}


###############################################################################
#  Turn on or off serial loopback for all channels in a PHY
###############################################################################

proc set_serial_loopback_phy_etile {phy serial_loop verbose} {
global offset_etile
global offset_etile_high_datarate
global high_datarate

				if {$verbose == 1} {	
				puts -nonewline "Phy : $phy  "
				puts -nonewline "	Turn on/off Serial Loopback on All channels" 
				puts ""
				}	

				 if {$high_datarate == 0} {
					set number_of_channels [rd_channel $phy 0 0x40010 0]
				} else {
					set temp [rd_channel $phy 0 0x40010 0]
					set number_of_channels [expr $temp/2]
					}
	


		
	for {set channel 0} {$channel < $number_of_channels} {incr channel} {
	
		set_loopback_etile $phy  $channel $serial_loop 0 $verbose

	}



}
puts {set_serial_loopback_phy_etile {phy serial_loop verbose}}
puts {example to turn serial loopback ON for all channels of etile_phy_33 :  set_serial_loopback_phy_etile etile_phy_33 1 1}
puts ""
if {$test_mode == 1} {
set_serial_loopback_phy_etile etile_phy_33 1 1
}




###############################################################################
#Perform run_prbs (needs to be checked as it does not work when NPHY is configured in KPFEC aggregrate mode)
###############################################################################
	
proc run_prbs {phy prbspattern rx_termination line_encoding tx_clk_divider rx_clk_divider gray_encoding encoding_1_1plusd serial_loop vod adaptation_mode verbose} {
global offset_etile
global offset_etile_high_datarate
global high_datarate

				if {$verbose == 1} {	
				puts -nonewline "Phy : $phy  "
				puts -nonewline "	Start PRBS-[format %d $prbspattern] generation and checking on all channels and run iadp on all channels" 
				puts ""
				}	
				

				 if {$high_datarate == 0} {
					set number_of_channels [rd_channel $phy 0 0x40010 0]
				} else {
					set temp [rd_channel $phy 0 0x40010 0]
					set number_of_channels [expr $temp/2]
					puts $number_of_channels
					}

			puts "channel "
		
	for {set channel 0} {$channel < $number_of_channels} {incr channel} {
	
	


	
		 puts -nonewline " [format %d $channel] "
 


#	//Set serial loopback
#	set_serial_loopback_etile (phy,j, 1);	

		set_loopback_etile $phy $channel $serial_loop 0 $verbose
		

	 
#	 //Set attenuation to 9 
#	  temp = 0x4000 + 9;  
#	  readback = rcfg_etile (phy,(j << CHANNEL_OFFSET), 0x15,temp,0x0015,1);	
#	  //printf("\nVOD write value : 0x%x , interrupt return value: 0x%x",tx_vodctrl[t][j],readback);

	 set temp [expr 0x4000 + $vod]
	 set readback_vod	[rcfg_etile $phy  $channel 0x15 $temp 0]
		

#	 //Configure line_encoding
#	

	#proc set_mode_etile {phy channel rx_termination line_encoding tx_clk_divider rx_clk_divider gray_encoding encoding_1_1plusd verbose}	
	set_mode_etile $phy $channel $rx_termination  $line_encoding $tx_clk_divider $rx_clk_divider $gray_encoding $encoding_1_1plusd $verbose  
	
	
	#	  //start prbspattern
#	  start_check_prbs_etile(phy,j,prbspattern);			
	
		start_check_prbs_etile $phy  $channel $prbspattern $verbose

#	//Set serial loopback
#	
#	set_serial_loopback_etile (phy,j, 1);	

#	set_serial_loopback_etile $phy $channel 1 1
	
		
	}
	after 1000
			
	puts ""
	
	iadp_phy $phy $adaptation_mode $verbose
	
	for {set channel 0} {$channel < $number_of_channels} {incr channel} {
	

#	if {$exit_mode == "OFF"} {
#	set temp	[rcfg_etile $phy $channel 0x02 0x3ff 0]
#		}
	}
	

}
#puts {run_prbs {phy prbspattern rx_termination line_encoding tx_clk_divider rx_clk_divider gray_encoding encoding_1_1plusd serial_loop vod adaptation_mode verbose}}
#if {$high_datarate == 0} {
#puts {example to run run_prbs with prbs31 with pam4 on all channels of etile_phy_33 in serial loopback with 0.5 effort adaptation and vod of 9 :  run_prbs etile_phy_33 31 1 PAM4 0x32 0x32 1 0 1 9 0.5 1}
#} else {
#puts {example to run run_prbs with prbs31 with pam4 on all channels of etile_phy_33 in serial loopback with 0.5 effort adaptation and vod of 9 :  run_prbs etile_phy_33 31 1 PAM4 0x64 0x64 1 0 1 9 0.5 1}
#}
#puts ""
#if {$test_mode == 1} {
#run_prbs etile_phy_33 31 1 PAM4 0x32 0x32 1 0 1 9 0.5 1 
#}


###############################################################################
#Start Prbs Generation and checking
###############################################################################

proc start_check_prbs_etile {phy channel pattern verbose} {
global offset_etile
global offset_etile_high_datarate
global high_datarate


			switch $pattern {
			7 {
			#set readback  rcfg_etile (phy,(channel << CHANNEL_OFFSET), 0x02,0x120,0x0002,1);	//Tx Pattern
			set readback  [rcfg_etile $phy  $channel 0x02 0x120 0]
			#set readback  rcfg_etile (phy,(channel << CHANNEL_OFFSET), 0x02,0x230,0x0002,1);	//Rx Pattern
			set readback  [rcfg_etile $phy  $channel 0x02 0x230 0]	
			}
			9 {
			#readback = rcfg_etile (phy,(channel << CHANNEL_OFFSET), 0x02,0x121,0x0002,1);	//Tx Pattern
			set readback  [rcfg_etile $phy  $channel 0x02 0x121 0]			
			#readback = rcfg_etile (phy,(channel << CHANNEL_OFFSET), 0x02,0x231,0x0002,1);	//Rx Pattern
			set readback  [rcfg_etile $phy  $channel 0x02 0x231 0]
			}
			11 {
			#readback = rcfg_etile (phy,(channel << CHANNEL_OFFSET), 0x02,0x122,0x0002,1);	//Tx Pattern
			set readback  [rcfg_etile $phy  $channel 0x02 0x122 0]			
			#readback = rcfg_etile (phy,(channel << CHANNEL_OFFSET), 0x02,0x233,0x0002,1);	//Rx Pattern
			set readback  [rcfg_etile $phy  $channel 0x02 0x233 0]
			}
			15 {
			#readback = rcfg_etile (phy,(channel << CHANNEL_OFFSET), 0x02,0x123,0x0002,1);	//Tx Pattern
			set readback  [rcfg_etile $phy  $channel 0x02 0x123 0]			
			#readback = rcfg_etile (phy,(channel << CHANNEL_OFFSET), 0x02,0x233,0x0002,1);	//Rx Pattern
			set readback  [rcfg_etile $phy  $channel 0x02 0x233 0]
			}
			23 {
			#readback = rcfg_etile (phy,(channel << CHANNEL_OFFSET), 0x02,0x124,0x0002,1);	//Tx Pattern
			set readback  [rcfg_etile $phy  $channel 0x02 0x124 0]			
			#readback = rcfg_etile (phy,(channel << CHANNEL_OFFSET), 0x02,0x234,0x0002,1);	//Rx Pattern
			set readback  [rcfg_etile $phy  $channel 0x02 0x234 0]
			}				
			31 {
			#readback = rcfg_etile (phy,(channel << CHANNEL_OFFSET), 0x02,0x125,0x0002,1);	//Tx Pattern
			set readback  [rcfg_etile $phy  $channel 0x02 0x125 0]			
			#readback = rcfg_etile (phy,(channel << CHANNEL_OFFSET), 0x02,0x235,0x0002,1);	//Rx Pattern
			set readback  [rcfg_etile $phy  $channel 0x02 0x235 0]
			}
			default {
			puts "Invalid pattern"
			}
		} 
		
   #Enable transceiver channel if is not running already.
   #readback = rcfg_etile (phy,(channel << CHANNEL_OFFSET), 0x01,0x0007,0x0001,1);
	set readback  [rcfg_etile $phy  $channel 0x01 0x0007 0]
  
	#set pattern configurator
   #readback = rcfg_etile (phy,(channel << CHANNEL_OFFSET), 0x03,0x0203,0x0003,1);
	set readback  [rcfg_etile $phy  $channel 0x03 0x0203 0]	
				
				# Print info

				if {$verbose == 1} {
				puts -nonewline "Phy : $phy  "
				puts -nonewline "channel  : $channel	"
				puts -nonewline "	Prbs Generation/Checking started with PRBS-$pattern" 
				puts ""				

}}
puts {start_check_prbs_etile {phy channel pattern verbose}}
puts {example to start prbs31 on channel 1 of etile_phy_33 :  start_check_prbs_etile etile_phy_33 1 31 1}
puts ""
if {$test_mode == 1} {
start_check_prbs_etile etile_phy_33 1 31 1
}


###############################################################################		
#Start Prbs Generation and checking on all channels of all Phys
###############################################################################
proc start_check_prbs_etile_all {pattern verbose} {
global ETILE_SLAVE_MAP
global offset_etile
global offset_etile_high_datarate
global high_datarate


set length [llength $ETILE_SLAVE_MAP]
puts $length 

	for {set i 0} {$i < $length/2} {incr i} {
	
	set phy [lindex $ETILE_SLAVE_MAP [expr $i * 2]]
	#puts $phy
				
			  if {$high_datarate == 0} {
					set number_of_channels [rd_channel $phy 0 0x40010 0]
				} else {
					set temp [rd_channel $phy 0 0x40010 0]
					set number_of_channels [expr $temp/2]
					#puts $number_of_channels
					}

					

			# Loop through all channels in this Native PHY
			for {set channel 0} {$channel < $number_of_channels} {incr channel} {
				

					switch $pattern {
					7 {
					#set readback  rcfg_etile (phy,(channel << CHANNEL_OFFSET), 0x02,0x120,0x0002,1);	//Tx Pattern
					set readback  [rcfg_etile $phy  $channel 0x02 0x120 0]
					#set readback  rcfg_etile (phy,(channel << CHANNEL_OFFSET), 0x02,0x230,0x0002,1);	//Rx Pattern
					set readback  [rcfg_etile $phy  $channel 0x02 0x230 0]	
					}
					9 {
					#readback = rcfg_etile (phy,(channel << CHANNEL_OFFSET), 0x02,0x121,0x0002,1);	//Tx Pattern
					set readback  [rcfg_etile $phy  $channel 0x02 0x121 0]			
					#readback = rcfg_etile (phy,(channel << CHANNEL_OFFSET), 0x02,0x231,0x0002,1);	//Rx Pattern
					set readback  [rcfg_etile $phy  $channel 0x02 0x231 0]
					}
					11 {
					#readback = rcfg_etile (phy,(channel << CHANNEL_OFFSET), 0x02,0x122,0x0002,1);	//Tx Pattern
					set readback  [rcfg_etile $phy  $channel 0x02 0x122 0]			
					#readback = rcfg_etile (phy,(channel << CHANNEL_OFFSET), 0x02,0x233,0x0002,1);	//Rx Pattern
					set readback  [rcfg_etile $phy  $channel 0x02 0x233 0]
					}
					15 {
					#readback = rcfg_etile (phy,(channel << CHANNEL_OFFSET), 0x02,0x123,0x0002,1);	//Tx Pattern
					set readback  [rcfg_etile $phy  $channel 0x02 0x123 0]			
					#readback = rcfg_etile (phy,(channel << CHANNEL_OFFSET), 0x02,0x233,0x0002,1);	//Rx Pattern
					set readback  [rcfg_etile $phy  $channel 0x02 0x233 0]
					}
					23 {
					#readback = rcfg_etile (phy,(channel << CHANNEL_OFFSET), 0x02,0x124,0x0002,1);	//Tx Pattern
					set readback  [rcfg_etile $phy  $channel 0x02 0x124 0]			
					#readback = rcfg_etile (phy,(channel << CHANNEL_OFFSET), 0x02,0x234,0x0002,1);	//Rx Pattern
					set readback  [rcfg_etile $phy  $channel 0x02 0x234 0]
					}				
					31 {
					#readback = rcfg_etile (phy,(channel << CHANNEL_OFFSET), 0x02,0x125,0x0002,1);	//Tx Pattern
					set readback  [rcfg_etile $phy  $channel 0x02 0x125 0]			
					#readback = rcfg_etile (phy,(channel << CHANNEL_OFFSET), 0x02,0x235,0x0002,1);	//Rx Pattern
					set readback  [rcfg_etile $phy  $channel 0x02 0x235 0]
					}
					default {
					puts "Invalid pattern"
					}
				} 
					
				#Enable transceiver channel if is not running already.
				#readback = rcfg_etile (phy,(channel << CHANNEL_OFFSET), 0x01,0x0007,0x0001,1);
				set readback  [rcfg_etile $phy $channel 0x01 0x0007 0]
			  
				#set pattern configurator
				#readback = rcfg_etile (phy,(channel << CHANNEL_OFFSET), 0x03,0x0203,0x0003,1);
				set readback  [rcfg_etile $phy $channel 0x03 0x0203 0]	
				
				# Print info

				if {$verbose == 1} {
				puts -nonewline "Phy : $phy  "
				puts -nonewline "channel  : $channel	"
				puts -nonewline "	Prbs Generation/Checking started with PRBS-$pattern" 
				puts ""						
				}
				
				
}}}
puts {start_check_prbs_etile_all {pattern verbose}}
puts {example useage to start prbs31 on all channels start_check_prbs_etile_all 31 1}
puts ""
if {$test_mode == 1} {
start_check_prbs_etile_all 31 1
}


###############################################################################		
#Continuosly check for errors on all etiles.
###############################################################################
proc check_prbserror_etile_all {verbose} {
global ETILE_SLAVE_MAP
global offset_etile
global offset_etile_high_datarate
global high_datarate


set length [llength $ETILE_SLAVE_MAP]
puts $length 

#Clear errorcount on all channels

	for {set i 0} {$i < $length/2} {incr i} {
	
	set phy [lindex $ETILE_SLAVE_MAP [expr $i * 2]]
	#puts $phy
				
			  if {$high_datarate == 0} {
					set number_of_channels [rd_channel $phy 0 0x40010 0]
				} else {
					set temp [rd_channel $phy 0 0x40010 0]
					set number_of_channels [expr $temp/2]
					#puts $number_of_channels
					}

					

			# Loop through all channels in this Native PHY
			for {set channel 0} {$channel < $number_of_channels} {incr channel} {
			
			reset_errorcount_etile $phy $channel 1
			}}
			
	set error_detected 0 			
	  
  while {$error_detected == 0} {
	for {set i 0} {$i < $length/2} {incr i} {
	
	set phy [lindex $ETILE_SLAVE_MAP [expr $i * 2]]
	#puts $phy
				
			  if {$high_datarate == 0} {
					set number_of_channels [rd_channel $phy 0 0x40010 0]
				} else {
					set temp [rd_channel $phy 0 0x40010 0]
					set number_of_channels [expr $temp/2]
					#puts $number_of_channels
					}

					

			# Loop through all channels in this Native PHY
			for {set channel 0} {$channel < $number_of_channels} {incr channel} {
				

			set errorcount [get_prbs_errorcount_etile $phy $channel $verbose]
			if {$errorcount != 0} {
			   set error_detected 1
			   puts "Error detected loop stopped"
				puts ""
				puts -nonewline "Phy : $phy  "
				puts -nonewline "channel  : $channel	"
				puts -nonewline " $errorcount biterrors detected" 
				puts ""
				}
				
				
}}}
}
puts {check_prbserror_etile_all {verbose} }
puts {example useage to continously check errorcount on all tiles  check_prbserror_etile_all 1}
puts ""



###############################################################################
#Inject PRBS error
###############################################################################

proc inject_error_etile {phy channel number_of_errors verbose} {
global offset_etile
global offset_etile_high_datarate
global high_datarate



			#readback = rcfg_etile (phy,(SelectedChannel << CHANNEL_OFFSET), 0x1b,errorcount,0x001B,1);
			set readback  [rcfg_etile $phy  $channel 0x1B $number_of_errors 0]
							
				# Print info

				if {$verbose == 1} {
				puts -nonewline "Phy : $phy  "
				puts -nonewline "channel  : $channel	"
				puts -nonewline " $number_of_errors biterrors inserted" 
				puts ""				
}}
puts {inject_error_etile {phy channel number_of_errors verbose}}
puts {example to insert 3 biterrors on channel 1 of etile_phy_33 :  inject_error_etile etile_phy_33 1 3 1}
puts ""
if {$test_mode == 1} {
inject_error_etile etile_phy_33 1 3 1
}


###############################################################################
#Reset PRBS errorcount
###############################################################################

proc reset_errorcount_etile {phy channel verbose} {
global offset_etile
global offset_etile_high_datarate
global high_datarate



			#readback = rcfg_etile (phy,(SelectedChannel << CHANNEL_OFFSET), 0x17,0x00,0x0017,1);
			set readback  [rcfg_etile $phy  $channel 0x17 0x00 0]
							
				# Print info

				if {$verbose == 1} {
				puts -nonewline "Phy : $phy  "
				puts -nonewline "channel  : $channel	"
				puts -nonewline " Errorcount resetted" 
				puts ""				
}}
puts {reset_errorcount_etile {phy channel verbose}}
puts {example to reset prbs errorcount on channel 1 of etile_phy_33 :  reset_errorcount_etile etile_phy_33 1 1}
puts ""
if {$test_mode == 1} {
reset_errorcount_etile etile_phy_33 1 1
}

###############################################################################
#Get PRBS errorcount
###############################################################################

proc get_prbs_errorcount_etile {phy channel verbose} {
global offset_etile
global offset_etile_high_datarate
global high_datarate



			#readback = rcfg_etile (phy,(SelectedChannel << CHANNEL_OFFSET), 0x18,0x03,0x0018,1);
			set readback  [rcfg_etile $phy  $channel 0x18 0x03 0]
			
			#read lower 16 bits
			#temp0 = rcfg_etile (phy,(SelectedChannel << CHANNEL_OFFSET), 0x1A,0x00,0x001A,0);
			set temp0  [rcfg_etile $phy  $channel 0x1A 0x00 0]
			
			#read upper 16 bits
			#temp0 = rcfg_etile (phy,(SelectedChannel << CHANNEL_OFFSET), 0x1A,0x00,0x001A,0);
			set temp1  [rcfg_etile $phy  $channel 0x1A 0x00 0]			
		
		   set ErrorCount    [expr $temp0 + ($temp1 * 65536)];
				
				# Print info

				if {$verbose == 1} {
				puts -nonewline "Phy : $phy  "
				puts -nonewline "channel  : $channel	"
				puts -nonewline " Errorcount : $ErrorCount" 
				puts ""				
return $ErrorCount
}}
puts {get_prbs_errorcount_etile {phy channel verbose}}
puts {example to get prbscount on channel 1 of etile_phy_33 :  get_prbs_errorcount_etile etile_phy_33 1 1}
puts ""
if {$test_mode == 1} {
get_prbs_errorcount_etile etile_phy_33 1 1
}


###############################################################################		
#Run Test : set serial loopback on all phys, run iADP on all phys and start prbs generation and checking
###############################################################################
proc run_test_all {pattern serial_loop adaptation_mode verbose} {
global ETILE_SLAVE_MAP
global offset_etile
global offset_etile_high_datarate
global high_datarate


set length [llength $ETILE_SLAVE_MAP]
puts $length 

# set loopback on all channels

	for {set i 0} {$i < $length/2} {incr i} {
	
	set phy [lindex $ETILE_SLAVE_MAP [expr $i * 2]]
	#puts $phy
				
			  if {$high_datarate == 0} {
					set number_of_channels [rd_channel $phy 0 0x40010 0]
				} else {
					set temp [rd_channel $phy 0 0x40010 0]
					set number_of_channels [expr $temp/2]
					#puts $number_of_channels
					}

			# Loop through all channels in this Native PHY
			for {set channel 0} {$channel < $number_of_channels} {incr channel} {		
			
				set_loopback_etile $phy $channel $serial_loop 0 $verbose	
				
				}
				
				}
				
				
# run adaptation on all channels
				
		for {set i 0} {$i < $length/2} {incr i} {
	
				set phy [lindex $ETILE_SLAVE_MAP [expr $i * 2]]
				#puts $phy			
				
				iadp_phy $phy $adaptation_mode $verbose
			}


# Start prbs check on all channels

start_check_prbs_etile_all $pattern 1

# Start prbs errorchecking

check_prbserror_etile_all 1

}
puts {run_test_all {pattern serial_loop adaptation_mode verbose}}
puts {example useage to start prbs31 on all channels with serial loopback and 0.5 iADP :  run_test_all 31 1 0.5 1}
puts ""


	
###############################################################################
# Function to readout firmware version
###############################################################################

proc get_firmware {phy} {

set firmware [rcfg_etile $phy 0 0x00 0x0000 0]
puts "firmware version Etile : $firmware"
}
puts {get_firmware {phy}}
puts {example to get firmware version :  get_firmware etile_phy_33}	
puts ""

###############################################################################
# The following are examples and can be used as a reference.
###############################################################################

# this will put all channels in serial loopback, run 0.5 iadp and start prbs checking on all channels 
proc start_test {} {
run_test_all 31 1 0.5 1
}


proc powerdown_serdes {phy verbose} {
global offset_etile
global offset_etile_high_datarate
global high_datarate

				if {$verbose == 1} {	
				puts -nonewline "Phy : $phy  "
				puts -nonewline "	Powerdown all serdes in PHY" 
				puts ""
				}	
				

				 if {$high_datarate == 0} {
					set number_of_channels [rd_channel $phy 0 0x40010 0]
				} else {
					set temp [rd_channel $phy 0 0x40010 0]
					set number_of_channels [expr $temp/2]
					#puts $number_of_channels
					}

			puts "channel "
		
	for {set channel 0} {$channel < $number_of_channels} {incr channel} {
	
	


	
		 puts -nonewline " [format %d $channel] "
 
		#disable serdes

			set readback  [rcfg_etile $phy  $channel 0x01 0x00 0]  

	}
}
puts {powerdown_serdes {phy verbose}}
puts {example to powerdown all channels of etile_phy_33  : powerdown_serdes etile_phy_33 1}
puts ""

proc powerup_serdes {phy verbose} {
global offset_etile
global offset_etile_high_datarate
global high_datarate

				if {$verbose == 1} {	
				puts -nonewline "Phy : $phy  "
				puts -nonewline "	Powerup all serdes in PHY" 
				puts ""
				}	
				

				 if {$high_datarate == 0} {
					set number_of_channels [rd_channel $phy 0 0x40010 0]
				} else {
					set temp [rd_channel $phy 0 0x40010 0]
					set number_of_channels [expr $temp/2]
					#puts $number_of_channels
					}

			puts "channel "
		
	for {set channel 0} {$channel < $number_of_channels} {incr channel} {
	
	


	
		 puts -nonewline " [format %d $channel] "
 
		#enable serdes

		set readback  [rcfg_etile $phy  $channel 0x01 0x07 0]  

	}
}
puts {powerup_serdes {phy verbose}}
puts {example to powerup all channels of etile_phy_33  : powerup_serdes etile_phy_33 1}
puts ""


###############################################################################
# Displays the PHY's found in the design
###############################################################################

#puts {}
#puts {These are the Native PHY's identified in the design : }
#puts {You can use the index of the phy to identify a certain phy }
#set phy [show_phys]

map_all_etile_slaves 1



puts ""
if {$high_datarate== 1} {
puts "IMPORTANT : OFFSET has been set for PAM4 high datarate, if this design does not use this, change the parameter high_datarate to 0"
} else {
puts "IMPORTANT : if design required PAM4 high datarate offsets, change the parameter high_datarate to 1"
}



